% Set whch to 1 to get the first plot(s) of the paper
% Then set to 2, 3 and 4 and get the rest. Takes a while
% And requires PLS_Toolbox
% And a fairly new version of MATLAB
whch=1
close all
if whch ==1
    % Simulated data, one run, plots the specific results for one run
    datset = 2;Factrs=2;plotit=1;reps = 1;RES=[];
elseif whch ==2
    % Simulated data, many run, plots just a boxplot summing up the above for many runs
    datset = 2;Factrs=2;plotit=0;reps = 500;
elseif whch ==3   
    % Real data, one run
    datset = 1;Factrs=6;plotit=1;reps = 1;RES=[];
elseif whch ==4
    % Real data, many runs
    datset = 1;Factrs=6;plotit=0;reps = 500;
end

for rep = 1:reps
    disp([num2str(rep),' of ',num2str(reps)])
    if datset==1
        if reps==1
            rng(2019)
        end
        load('nmr')
        id1 = randperm(419);
        id2 = randperm(419)+419;
        calnum = 120;
        
        Xs = dataset(X);
        Xs.class{1}=Clas;
        Xs = Xs(:,~isnan(sum(Xs.data)));
        Xtst = Xs([id1(calnum+1:end);id2(calnum+1:end)],:);
        Xs = Xs([id1(1:calnum);id2(1:calnum)],:);
    elseif datset ==2
        if reps == 1 % Then we are doing THE model for the paper,
            % hence fix the random numbers. Otherwise we are
            % doing replicates.
            rng(2019);
        end
        close all
        clear X
        T = rand(2000,1);
        T(1:size(T,1)/2,1)=.3;
        T(size(T,1)/2+1:end,1)=.7;
        T = T+randn(size(T))*.1;
        P = rand(10,size(T,2));
        P2 = rand(350,size(T,2));
        T2 = 2*rand(size(T));
        X = [T*P' T2*P2'];
        
        X = X+randn(size(X))*.2;
        X = dataset(X);
        X.class{1}=round(T(:,1))+1;
        id = 1:50:size(X,1);
        Xs = X(id,:);
        Xtst = X;
        Xtst(id,:)=[];
        if rep==1
            t = [T T2];
            for iu=1:2
                subplot(2,1,iu)
                plot(t(:,iu));
                axis tight
                kk = max(t(:));
                if iu==1
                    j=patch([0 0 1000 1000],[0 kk kk 0],'red','facealpha',.2);
                    j2=patch([1001 1001 2000 2000],[0 kk kk 0],'blue','facealpha',.2);
                    text(300,1.2,'Class 1','Color',[.8 .1 .3],'Fontweight','bold');
                    text(1300,1.5,'Class 2','Color',[.1 .3 .8],'Fontweight','bold');
                end
                set(gca,'children',flipud(get(gca,'children')));
                xlabel(['sample #'])
                ylabel(['Score ',num2str(iu)])
            end
            shg
        end
    end
    
    % Calibrate
    res=[];
    s = preprocess('default','autoscale');
    [Dp,sp] = preprocess('calibrate',s,Xs);
    [Dp2] = preprocess('apply',sp,Xtst);
    
    ytrue = mncn(Dp.class{1}');
    ytruetest = Dp2.class{1}'-mean(Dp.class{1}');
    model = pls(Dp,ytrue,Factrs,struct('plots','off','display','off'));
    modelO = pls(Dp,ytrue,Factrs,struct('plots','off','orthogonalize','on','display','off'));
    modeltest = pls(Dp2,model,struct('plots','off','display','off'));
    
    % Plot and compare
    yp = model.pred{2}(:,1);
    T = dataset(model.scores);
    T.class{1}=Xs.class{1};
    R2 = round(100*(1-ssq(ytrue-yp)/ssq(ytrue)));
    col = 'rb';
    fs = 36;
    
    if plotit
        figure
        nexttile
        cols{1}=[1 .7 .7];
        cols{2}=[.7 .7 1];
        for j=1:2
            id = T.class{1}==j;
            scatter1 = scatter(T.data(id,1),T.data(id,2),'sizedata',500,'MarkerFaceColor',col(j),'MarkerEdgeColor',col(j));
            scatter1.MarkerFaceAlpha = .2;scatter1.MarkerEdgeAlpha = .2;
            hold on
        end
        ylabel('Score 2')
        title('PLS-DA calibration','fontsize',fs)
        drawnow
    end
    res=[res R2];
    
    TO = dataset(modelO.scores);
    TO.class{1}=Xs.class{1};
    if plotit
        nexttile
        for j=1:2
            id = TO.class{1}==j;
            scatter1 = scatter(TO.data(id,1),TO.data(id,2),'sizedata',500,'MarkerFaceColor',col(j),'MarkerEdgeColor',col(j));
            scatter1.MarkerFaceAlpha = .2;scatter1.MarkerEdgeAlpha = .2;
            hold on
        end
        title('OPLS-DA calibration','fontsize',fs)
        drawnow
    end
    
    [Tc,ypred2]=crossit(Dp,Factrs,s);
    Tc = dataset(Tc);
    Tc.class{1}=Xs.class{1};
    Q2 = round(100*(1-ssq(ytrue-ypred2)/ssq(ytrue)));
    if plotit
        nexttile
        for j=1:2
            id = Tc.class{1}==j;
            scatter1 = scatter(Tc.data(id,1),Tc.data(id,2),'sizedata',500,'MarkerFaceColor',col(j),'MarkerEdgeColor',col(j));
            scatter1.MarkerFaceAlpha = .2;scatter1.MarkerEdgeAlpha = .2;
            hold on
        end
        ylabel('Score 2')
        xlabel('Score 1')
        title(['Crossvalidation'],'fontsize',fs)
        drawnow
    end
    res=[res Q2];
    
    ytst = modeltest.pred{2}(:,1);
    Ttst = dataset(modeltest.scores);
    Ttst.class{1}=Xtst.class{1};
    Qtst2 = round(100*(1-ssq(ytruetest-ytst)/ssq(ytruetest)));
    if plotit
        nexttile
        for j=1:2
            id = Ttst.class{1}==j;
            scatter1 = scatter(Ttst.data(id,1),Ttst.data(id,2),'sizedata',500,'MarkerFaceColor',col(j),'MarkerEdgeColor',col(j));
            scatter1.MarkerFaceAlpha = .2;scatter1.MarkerEdgeAlpha = .2;
            hold on
        end
        title('Test set','fontsize',fs)
        xlabel('Score 1')
        drawnow
    end
    res=[res Qtst2];
    RES=[RES;res];
end

if plotit
    figure
    bar(RES);
    RES2 = RES; % So I can plot together with resampled when running that
    ylabel('Variance explained')
    set(gca,'XTickLabel',{'Cal','Crossval','Test'},'xticklabelrotation',45,'fontweight','bold')
    title('(R^2/Q^2)','fontsize',fs)
end

%
if size(RES,1)>1
    subplot(1,2,1)
    bar(RES2/100)
    ylabel('R^2 and Q^2')
    set(gca,'XTickLabel',{'Cal','Crossval','Test'},'xticklabelrotation',45,'fontweight','bold')
    axis([.5 3.5 0 1])
    
    
    h=subplot(1,2,2)
    boxplot(h,RES/100,'notch','on'),shg
    set(gca,'xticklabel',{'Cal';'CV';'Test'})
    ylabel('R^2 and Q^2')
    set(gca,'XTickLabel',{'Cal','Crossval','Test'},'xticklabelrotation',45,'fontweight','bold')
    axis([.7 3.3 0 1])
    save dataset_2
    legend
end



